#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
    /// A modification of the SuperTrend indicator that is based on fibonacci retracements
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
	public class FibonacciSuperTrend : Indicator
	{
		#region Variables
        private int trend;
		private Series<bool> Trend;
		private double hh = 0, ll = 0;
		private MIN min;
		private MAX max;
        #endregion
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Fibonacci SuperTrend Indicator from www.FreeIndicators.com";
				Name										= "FibonacciSuperTrend";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= false;
				
				Length						= 14;
				Retrace						= 38.2;
				ShowArrows					= true;
				UseHighLow					= true;
				ColorBars					= true;
				AudioAlerts					= true;
				AlertFile					= "Alert1.wav";
				
				AddPlot(Brushes.Green, "UpTrend");
				AddPlot(Brushes.Red, "DownTrend");
			}
			else if (State == State.DataLoaded)
			{
				ISeries<double> h = UseHighLow ? High : Close;
				ISeries<double> l = UseHighLow ? Low : Close;
				
				min = MIN(l,Length);
				max = MAX(h,Length);
				
				Trend = new Series<bool>(this);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar<1)
				return;
			
			double h = UseHighLow ? High[0] : Close[0];
			double l = UseHighLow ? Low[0] : Close[0];
			
			if (h>max[1])
			{
				hh=h;
				trend=1;
			}
			else if (l<min[1])
			{
				ll=l;
				trend=-1;
			}
				
			Trend[0] = trend==1;
			
			if(Trend[0] && !Trend[1])
			{
				UpTrend[0] = SellPrice;
				UpTrend[1] = DownTrend[1];
				if (ShowArrows)
					Draw.ArrowUp(this,CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize, UpColor);
				if (AudioAlerts)
					PlaySound(AlertFile);	
			} 
			else if (!Trend[0]  && Trend[1])
			{
				DownTrend[0] = BuyPrice;
				DownTrend[1] = UpTrend[1];
				if (ShowArrows)
					Draw.ArrowDown(this,CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize, DownColor);
				if (AudioAlerts)
					PlaySound(AlertFile);	
			} 
			else if (Trend[0])
				UpTrend[0] = (SellPrice > UpTrend[1] ? SellPrice : UpTrend[1]);
			else
				DownTrend[0] = (BuyPrice < DownTrend[1] ? BuyPrice : DownTrend[1]);
													
			// Color bars 
			if (ColorBars)
			{
				if (trend==1)
					BarBrush=UpColor;
				else if (trend==-1)									
					BarBrush=DownColor;				
				else 
					BarBrush=null;
			}	
		}
		
		private Brush UpColor
		{
			get { return Plots[0].Pen.Brush; }
		}
		
		private Brush DownColor
		{
			get { return Plots[1].Pen.Brush; }
		}
		
		private double MyRange
		{
			get { return hh-ll; }						
		}
	
		private double BuyPrice
		{
			get { return ll+Retracement*MyRange; }						
		}
		
		private double SellPrice
		{
			get { return hh-Retracement*MyRange; }			
		}
		
		private double Retracement
		{
			get {return Retrace/100;}			
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Length", Order=1, GroupName="Parameters")]
		public int Length
		{ get; set; }

		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="Retrace", Order=2, GroupName="Parameters")]
		public double Retrace
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="ShowArrows", Order=3, GroupName="Parameters")]
		public bool ShowArrows
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="UseHighLow", Order=4, GroupName="Parameters")]
		public bool UseHighLow
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="ColorBars", Order=5, GroupName="Parameters")]
		public bool ColorBars
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="AudioAlerts", Order=6, GroupName="Parameters")]
		public bool AudioAlerts
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="AlertFile", Order=7, GroupName="Parameters")]
		public string AlertFile
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> UpTrend
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> DownTrend
		{
			get { return Values[1]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private FibonacciSuperTrend[] cacheFibonacciSuperTrend;
		public FibonacciSuperTrend FibonacciSuperTrend(int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			return FibonacciSuperTrend(Input, length, retrace, showArrows, useHighLow, colorBars, audioAlerts, alertFile);
		}

		public FibonacciSuperTrend FibonacciSuperTrend(ISeries<double> input, int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			if (cacheFibonacciSuperTrend != null)
				for (int idx = 0; idx < cacheFibonacciSuperTrend.Length; idx++)
					if (cacheFibonacciSuperTrend[idx] != null && cacheFibonacciSuperTrend[idx].Length == length && cacheFibonacciSuperTrend[idx].Retrace == retrace && cacheFibonacciSuperTrend[idx].ShowArrows == showArrows && cacheFibonacciSuperTrend[idx].UseHighLow == useHighLow && cacheFibonacciSuperTrend[idx].ColorBars == colorBars && cacheFibonacciSuperTrend[idx].AudioAlerts == audioAlerts && cacheFibonacciSuperTrend[idx].AlertFile == alertFile && cacheFibonacciSuperTrend[idx].EqualsInput(input))
						return cacheFibonacciSuperTrend[idx];
			return CacheIndicator<FibonacciSuperTrend>(new FibonacciSuperTrend(){ Length = length, Retrace = retrace, ShowArrows = showArrows, UseHighLow = useHighLow, ColorBars = colorBars, AudioAlerts = audioAlerts, AlertFile = alertFile }, input, ref cacheFibonacciSuperTrend);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.FibonacciSuperTrend FibonacciSuperTrend(int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			return indicator.FibonacciSuperTrend(Input, length, retrace, showArrows, useHighLow, colorBars, audioAlerts, alertFile);
		}

		public Indicators.FibonacciSuperTrend FibonacciSuperTrend(ISeries<double> input , int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			return indicator.FibonacciSuperTrend(input, length, retrace, showArrows, useHighLow, colorBars, audioAlerts, alertFile);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.FibonacciSuperTrend FibonacciSuperTrend(int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			return indicator.FibonacciSuperTrend(Input, length, retrace, showArrows, useHighLow, colorBars, audioAlerts, alertFile);
		}

		public Indicators.FibonacciSuperTrend FibonacciSuperTrend(ISeries<double> input , int length, double retrace, bool showArrows, bool useHighLow, bool colorBars, bool audioAlerts, string alertFile)
		{
			return indicator.FibonacciSuperTrend(input, length, retrace, showArrows, useHighLow, colorBars, audioAlerts, alertFile);
		}
	}
}

#endregion
